package com.care.installation.controller;


import com.care.common.util.Result;
import com.care.installation.service.DevInstallService;
import com.care.installation.service.RoomService;
import com.care.installation.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * 描述：
 *
 * @autho：wenskys
 * @create 2020/5/12 14:13
 **/
@Api(tags = "设备安装接口")
@Slf4j
@RestController
@RequestMapping("/installation/api")
public class RoomController {

    @Resource
    private RoomService roomService;


    /**
     * 获取是否配置的状态接口
     * @param devCode
     * @return
     */
    @GetMapping("getConfStatusInfo")
    @ApiOperation(tags = {"设备安装"},value = "获取是否配置的状态接口")
    public Result<Map<String,Boolean>> getConfStatusInfo(@RequestParam("devCode") String devCode) {
        return Result.success(this.roomService.getConfStatusInfo(devCode));
    }

    /**
     * 获取房间信息详情
     * @param devCode
     * @return
     */
    @GetMapping("getRoomInfo")
    @ApiOperation(tags = {"设备安装"},value = "获取房间信息详情")
    public Result<RoomVO> getRoomInfo(@RequestParam("devCode") String devCode) {
        return Result.success(this.roomService.getRoomInfo(devCode));
    }

    /**
     * 保存房间信息
     * @return
     */
    @PostMapping("saveRoom")
    @ApiOperation(tags = {"设备安装"},value = "保存房间信息，注意：roomId（取值devCode）必须传；id空是新增，id有是修改")
    public Result<Object> saveRoom(@RequestBody RoomVO vo){
        if (this.roomService.saveRoom(vo)){
            return  Result.success("保存成功");
        } else{
            return  Result.error("保存失败");
        }
    }

    /**
     * 获取传感器信息详情
     * @param devCode
     * @return
     */
    @GetMapping("getRoomSensorInfo")
    @ApiOperation(tags = {"设备安装"},value = "获取传感器信息详情")
    public Result<RoomSensorVO> getRoomSensorInfo(@RequestParam("devCode") String devCode) {
        return Result.success(this.roomService.getRoomSensorInfo(devCode));
    }

    /**
     * 保存传感器信息
     * @return
     */
    @PostMapping("saveRoomSensor")
    @ApiOperation(tags = {"设备安装"},value = "保存传感器信息，注意：roomId（取值devCode）必须传；id空是新增，id有是修改")
    public Result<Object> saveRoomSensor(@RequestBody RoomSensorVO vo){
        if (this.roomService.saveRoomSensor(vo)){
            return  Result.success("保存成功");
        } else{
            return  Result.error("保存失败");
        }
    }

    /**
     * 获取门信息列表
     * @param devCode
     * @return
     */
    @GetMapping("listRoomGateInfo")
    @ApiOperation(tags = {"设备安装"},value = "获取门信息列表")
    public Result<List<RoomGateVO>> listRoomGateInfo(@RequestParam("devCode") String devCode) {
        return Result.success(this.roomService.listRoomGateInfo(devCode));
    }

    /**
     * 获取门信息详情
     * @param devCode
     * @return
     */
    @GetMapping("getRoomGateInfo")
    @ApiOperation(tags = {"设备安装"},value = "获取门信息详情")
    public Result<RoomGateVO> getRoomGateInfo(@RequestParam("devCode") String devCode, @RequestParam("gid") Integer gid) {
        return Result.success(this.roomService.getRoomGateInfo(devCode,gid));
    }

    /**
     * 保存门信息
     * @return
     */
    @PostMapping("saveRoomGate")
    @ApiOperation(tags = {"设备安装"},value = "保存门信息，注意：roomId（取值devCode）必须传；id空是新增，id有是修改")
    public Result<Object> saveRoomGate(@RequestBody RoomGateVO vo){
        if (this.roomService.saveRoomGate(vo)){
            return  Result.success("保存成功");
        } else{
            return  Result.error("保存失败");
        }
    }

    @DeleteMapping("delRoomGate")
    @ApiOperation(tags = {"设备安装"},value = "删除门信息")
    public Result<Object> delRoomGate(@RequestParam("devCode") String devCode, @RequestParam("gid") Integer gid){
        try {
            this.roomService.delRoomGate(devCode, gid);
            return  Result.success("删除成功！");
        } catch (Exception e) {
            log.error("设备安装: 删除门出现异常",e);
            return Result.error("删除失败！");
        }
    }

    /**
     * 获取区域信息列表
     * @param devCode
     * @return
     */
    @GetMapping("listRoomRegionInfo")
    @ApiOperation(tags = {"设备安装"},value = "获取区域信息列表")
    public Result<List<RoomRegionVO>> listRoomRegionInfo(@RequestParam("devCode") String devCode) {
        return Result.success(this.roomService.listRoomRegionInfo(devCode));
    }

    /**
     * 获取区域信息详情
     * @param devCode
     * @return
     */
    @GetMapping("getRoomRegionInfo")
    @ApiOperation(tags = {"设备安装"},value = "获取门信息详情")
    public Result<RoomRegionVO> getRoomRegionInfo(@RequestParam("devCode") String devCode, @RequestParam("rid") Integer rid) {
        return Result.success(this.roomService.getRoomRegionInfo(devCode,rid));
    }

    /**
     * 保存区域信息
     * @return
     */
    @PostMapping("saveRoomRegion")
    @ApiOperation(tags = {"设备安装"},value = "保存区域信息，注意：roomId（取值devCode）必须传；id空是新增，id有是修改")
    public Result<Object> saveRoomRegion(@RequestBody RoomRegionVO vo){
        if (this.roomService.saveRoomRegion(vo)){
            return  Result.success("保存成功");
        } else{
            return  Result.error("保存失败");
        }
    }

    @DeleteMapping("delRoomRegion")
    @ApiOperation(tags = {"设备安装"},value = "删除区域信息")
    public Result<Object> delRoomRegion(@RequestParam("devCode") String devCode, @RequestParam("rid") Integer rid){
        try {
            this.roomService.delRoomRegion(devCode, rid);
            return  Result.success("删除成功！");
        } catch (Exception e) {
            log.error("设备安装: 删除区域出现异常",e);
            return Result.error("删除失败！");
        }
    }


    /**
     * 提交房间信息
     * @return
     */
    @PostMapping("commitRoomInfo")
    @ApiOperation(tags = {"设备安装"},value = "提交提交房间信息")
    public Result<Object> commitRoomInfo(@RequestParam("devCode") String devCode) {
        if (this.roomService.commitRoomInfo(devCode)){
            return  Result.success("提交成功");
        } else{
            return  Result.error("提交失败");
        }
    }

    /**
     * 查询配置房间信息的结果
     * @return
     */
    @GetMapping("getCommitRoomInfoResult")
    @ApiOperation(tags = {"设备安装"},value = "查询配置房间信息的结果,0：处理中，1：成功，2：失败,3:超时")
    public Result<String> getConfigureRoomInfoResult(@RequestParam("devCode") String devCode) {
        return Result.success("请求成功",this.roomService.getConfigureRoomInfoResult(devCode));
    }

}
