package com.care.installation.service;


import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.care.common.entity.*;

import com.care.common.service.*;

import com.care.common.util.CommonConfUtil;
import com.care.installation.vo.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.UUID;

@Slf4j
@Service
public class DevInstallService {


    @Resource
    private CareDeviceService careDeviceService;
    @Resource
    private CareHouseService careHouseService;
    @Resource
    private CareMemberInfoService careMemberInfoService;


    public Boolean checkDevCode(String devCode) {
        QueryWrapper<CareDevice> queryWrapper = new QueryWrapper<>();
       queryWrapper.lambda().eq(CareDevice::getDevCode,devCode);
        Integer cnt = careDeviceService.count(queryWrapper);
        if ( cnt != null && cnt > 0) {
            return true;
        } else{
            return false;
        }

    }

    public DeviceInstallationVO getDevInstallationInfo(String devCode) {
        QueryWrapper<CareDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareDevice::getDevCode,devCode);
        CareDevice careDevice = careDeviceService.getOne(queryWrapper);
        if(careDevice != null) {
            DeviceInstallationVO vo = new DeviceInstallationVO();
            BeanUtil.copyProperties(careDevice,vo);

            //房屋地址
            if (careDevice.getHouseId() != null) {
                CareHouse careHouse = careHouseService.getById(careDevice.getHouseId());
                if (careHouse != null) {
                    vo.setHouseId(careDevice.getHouseId());
                    vo.setHouseAddr(careHouse.getAddr());
                    vo.setLatitude(careHouse.getLatitude());
                    vo.setLongitude(careHouse.getLongitude());
                }
             }

            return vo;
        }
        return null;
    }

    public Boolean saveDevInstallation(DeviceInstallationVO vo) {
        if(StringUtils.isEmpty(vo.getDevCode())){
            return false;
        }

       CareDevice careDevice = new CareDevice();
        BeanUtil.copyProperties(vo,careDevice);
        Date date = new Date();
        careDevice.setModifyTime(date);
        if(careDevice.getId() == null){
            QueryWrapper<CareDevice> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(CareDevice::getDevCode,vo.getDevCode());
            CareDevice careDeviceDb = careDeviceService.getOne(queryWrapper);
            careDevice.setId(careDeviceDb.getId());
            careDevice.setMemberId(careDeviceDb.getMemberId());
            careDevice.setOrgId(careDeviceDb.getOrgId());
            careDevice.setStationId(careDeviceDb.getStationId());
        }

//        //关联用户
//        if(StringUtils.isNotEmpty(vo.getMemberPhone())) {
//            QueryWrapper<CareMemberInfo> queryWrapper0 = new QueryWrapper<>();
//            queryWrapper0.lambda().eq(CareMemberInfo::getPhone,vo.getMemberPhone());
//            CareMemberInfo careMemberInfo = this.careMemberInfoService.getOne(queryWrapper0);
//            if (careMemberInfo != null) { //关联,
//                careDevice.setMemberId(careMemberInfo.getId());
//            }
//        }

        //房屋地址
        if(StringUtils.isNotEmpty(vo.getHouseAddr())) {
            CareHouse careHouse = new CareHouse();
            careHouse.setId(vo.getHouseId());
            careHouse.setName(vo.getHouseAddr());
            careHouse.setAddr(vo.getHouseAddr());
            careHouse.setLongitude(vo.getLongitude());
            careHouse.setLatitude(vo.getLatitude());
            if(careDevice.getMemberId() != null && careDevice.getMemberId() != 0L){
                careHouse.setMemberId(careDevice.getMemberId());
            }
            if(careHouse.getId() != null) { //修改
                careHouse.setModifyTime(date);
                this.careHouseService.updateById(careHouse);
            } else { //新增
                careHouse.setOrgId(careDevice.getOrgId());
                careHouse.setStationId(careDevice.getStationId());
                careHouse.setCreateTime(date);
                this.careHouseService.save(careHouse);

                careDevice.setHouseId(careHouse.getId());
            }
        }
        //mqtt
        if(StringUtils.isEmpty(careDevice.getTopic())){
            String topic = CommonConfUtil.getConf("mqtt.topic.work.template").replace("XXXXX",careDevice.getDevCode());
            careDevice.setTopic(topic);
        }
        if(StringUtils.isEmpty(careDevice.getTopicInstall())){
            String topicInstall = CommonConfUtil.getConf("mqtt.topic.install.template").replace("XXXXX",careDevice.getDevCode());
            careDevice.setTopicInstall(topicInstall);
        }
        if(StringUtils.isEmpty(careDevice.getClientId())){
            careDevice.setClientId(UUID.randomUUID().toString());
        }
        if(StringUtils.isEmpty(careDevice.getClientIdInstall())){
            careDevice.setClientIdInstall(UUID.randomUUID().toString());
        }

        return careDeviceService.updateById(careDevice);
    }

}
