package com.care.mqtt.service;


import com.care.common.entity.CareDevice;
import com.care.common.util.CommonConfUtil;
import com.care.mqtt.tool.MqttDataConnector;
import com.care.mqtt.tool.MqttDataConnectorCache;
import com.care.mqtt.tool.MqttMessageListener;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

/**
 * @Author: stw
 * @Date: 2021/8/14
 * @Desc:
 */
@Service
public class MqttConnectorInstallPoolService {
    @Resource
    private MqttMsgInstallService mqttMsgInstallService;


    /**
     * 添加一个设备到服务
     */
    public MqttDataConnector addInsallOne(CareDevice careDevice) {

        if (StringUtils.isEmpty(careDevice.getTopicInstall()) || StringUtils.isEmpty(careDevice.getClientIdInstall())) {
            return null;
        }

        String cliendIdInstall = careDevice.getClientIdInstall();

        MqttDataConnector xbMqttDataConnector = MqttDataConnectorCache.getMqttDataConnectorMap().get(cliendIdInstall);
        if ( xbMqttDataConnector != null) {
            return xbMqttDataConnector;
        }

        xbMqttDataConnector = new MqttDataConnector();
        Map<String, String> configParams = new HashMap<>();
        String host = "ws://" + CommonConfUtil.getConf("mqtt.broker") + ":" + CommonConfUtil.getConf("mqtt.port");

        String username = CommonConfUtil.getConf("mqtt.username");
        String password = CommonConfUtil.getConf("mqtt.password");
        String topic = careDevice.getTopic();
        configParams.put("MQTT_HOST", host);
        configParams.put("MQTT_CLIENTID", cliendIdInstall);
        configParams.put("MQTT_USERNAME", username);
        configParams.put("MQTT_PASSWORD", password);
        configParams.put("CLEAN_SESSION", "true");

        try {
            xbMqttDataConnector.createIotDataSource(configParams);
            MqttMessageListener mqttMessageListener = new MqttMessageListener();
            mqttMessageListener.setMqttMsgInstallService(mqttMsgInstallService);
            xbMqttDataConnector.setReSubscribe(topic, mqttMessageListener);

            MqttDataConnectorCache.getMqttDataConnectorMap().put(cliendIdInstall, xbMqttDataConnector);

            return xbMqttDataConnector;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 从服务中去除一个设备
     */
    public void removeInstallOne(CareDevice careDevice) {
        String clientIdInstall = careDevice.getClientIdInstall();
        if (StringUtils.isEmpty(clientIdInstall)) {
            return;
        }
        if (MqttDataConnectorCache.getMqttDataConnectorMap().get(clientIdInstall) != null) {
            MqttDataConnectorCache.getMqttDataConnectorMap().get(clientIdInstall).destroy();
            MqttDataConnectorCache.getMqttDataConnectorMap().remove(clientIdInstall);
        }
    }

}
